﻿using System;
using System.Collections.Generic;
using System.DirectoryServices;
using EleWise.ELMA.ComponentModel;
using EleWise.ELMA.LdapSample.Services;
using EleWise.ELMA.LdapSample.Settings;
using EleWise.ELMA.Logging;
using EleWise.ELMA.Security;
using EleWise.ELMA.Security.Components;
using EleWise.ELMA.Security.Services;
using EleWise.ELMA.Security.Settings;
using EleWise.ELMA.Services;

namespace EleWise.ELMA.LdapSample.Components
{
    /// <summary>
    /// Базовый класс внешнего модуля авторизации LDAP
    /// </summary>
    [Component]
    public class LdapExternalMembershipModule : ExternalMembershipModule<LdapExternalMembershipModuleSettings>
    {
        private static readonly ILogger IntegrationLdapLog = Logger.GetLogger("ElmaIntegrationLdap");

        /// <summary>
        /// Уникальный номер модуля
        /// </summary>
        public override Guid Uid { get { return new Guid("8d9b3158-03cc-4a7e-b600-81b5f69461ae"); } }
        //NOTE номер модуля должен быть уникальный, необходимо указать новый Guid 

        /// <summary>
        /// Отображаемое имя
        /// </summary>
        public override string Name
        {
            get { return ("LDAP Sample"); }
        }

        /// <summary>
        /// Описание
        /// </summary>
        public override string Description
        {
            get { return SR.T("Модуль авторизации для произвольного LDAP Sample"); }
        }

        /// <summary>
        /// Группа модулей
        /// </summary>
        public override string Category
        {
            //NOTE Нужно изменить название, если необходима отдельная группа серверов
            get { return SR.T("Серверы LDAP Sample"); }
        }

        ///<summary>
        /// Поиск пользователей по подстроке
        ///</summary>
        public override IEnumerable<IUser> FindUsers(string searchString, IExternalMembershipModuleSettings settings)
        {
            var service = Locator.GetServiceNotNull<ILdapService>();
            return service.FindUsers(searchString, settings as LdapExternalMembershipModuleSettings, settings.InstanceUid);
        }

        ///<summary>
        /// Синхронизировать данные о пользователе
        ///</summary>
        public override void Sync(IUser user, IExternalMembershipModuleSettings settings)
        {
            var service = Locator.GetServiceNotNull<ILdapService>();
            service.Sync(user, settings as LdapExternalMembershipModuleSettings);
        }

        /// <summary>
        /// Проверить авторизацию пользователя
        /// </summary>
        public override bool ValidateUser(UserValidationContext context, IExternalMembershipModuleSettings settings)
        {
            var service = Locator.GetServiceNotNull<ILdapService>();

            //попытка авторизации по DN
            try
            {
                if (!service.ValidatingDN(context, settings as LdapExternalMembershipModuleSettings))
                    return false;
            }
            catch (Exception ex1)
            {
                //попытка авторизации по шаблону
                try
                {
                    if (!service.ValidatingTemplate(context, settings as LdapExternalMembershipModuleSettings))
                        return false;
                }
                catch (Exception ex2)
                {
                    IntegrationLdapLog.Error(string.Format("LDAP DN fail. Login: {0}. Exception: {1}", context.User.UserName, ex1.Message), ex1);
                    IntegrationLdapLog.Error(string.Format("LDAP Template fail. Login: {0}. Exception: {1}", context.User.UserName, ex2.Message), ex2);
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Создать экземпляр модуля с настройками по умолчанию
        /// </summary>
        public override IExternalMembershipModuleSettings CreateDefaultSettings()
        {
            var settings = base.CreateDefaultSettings() as LdapExternalMembershipModuleSettings;
            if (settings != null)
            {
                settings.LdapUrl = "127.0.0.1";
                settings.LdapAuthType = (int)AuthenticationTypes.None;
                settings.LdapAuthFilter = "(objectClass=person)";
                settings.LdapTelephoneNumber = "telephoneNumber";
            }
            return settings;
        }

    }

}